/*
 * (c) Copyright 2004, 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * [See end of file]
 */

package com.hp.hpl.jena.query.engine;

import java.util.* ;

import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.graph.Node ;

import com.hp.hpl.jena.query.core.ARQInternalErrorException;
import com.hp.hpl.jena.query.core.Var;
import com.hp.hpl.jena.query.util.FmtUtils;
import com.hp.hpl.jena.util.iterator.SingletonIterator ;


/** Special purpose binding for adding just one name/value slot. 
 * 
 * @author   Andy Seaborne
 * @version  $Id: Binding1.java,v 1.3 2007/01/02 11:19:35 andy_seaborne Exp $
 */


public class Binding1 extends BindingBase
{
    Var var ;
    Node value ;
    
    public Binding1(Binding parent, Var _var, Node _node)
    { 
        super(parent) ;
        checkAdd1(_var, _node) ;
        var = _var ; 
        value = _node ;
    }
    
    protected void add1(Var v, Node node)
    {
        throw new UnsupportedOperationException("Binding1.add1") ;
    }

    
    /** Iterate over all the names of variables.
     */
    public Iterator vars1() 
    {
        return new SingletonIterator(var) ;
    }
    
    public boolean contains1(Var n)
    {
        return var.equals(n) ;
    }
    
    public Node get1(Var v)
    {
        if ( v.equals(var) )
            return value ;
        return null ;
    }

    protected void checkAdd1(Var v, Node val)
    {
        if ( parent == null ) return ;
        if ( ! parent.contains(v) ) return ;

        if ( parent.get(v).equals(val) )
            LogFactory.getLog(Binding1.class).
            warn("Attempt to reassign '"+v+"' to '"+FmtUtils.stringForNode(val)+"'") ;
        else
            throw new ARQInternalErrorException("Attempt to reassign '"+v+
                                                "' from '"+FmtUtils.stringForNode(parent.get(v))+
                                                "' to '"+FmtUtils.stringForNode(val)+"'") ;
    }
}

/*
 *  (c) Copyright 2004, 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
