/*
 * (c) Copyright 2004, 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * [See end of file]
 */

package com.hp.hpl.jena.query.engine;

import java.util.Iterator;

import com.hp.hpl.jena.graph.Node ;
import com.hp.hpl.jena.query.core.Var;
import com.hp.hpl.jena.util.iterator.Filter;
import com.hp.hpl.jena.util.iterator.FilterDropIterator;

/** Special purpose binding for the root of all bindings.
 *  Contains system information and initial, pre-bound, solutions
 * 
 * @author   Andy Seaborne
 * @version  $Id: BindingRoot.java,v 1.3 2007/01/02 11:19:35 andy_seaborne Exp $
 */


public class BindingRoot extends BindingMap 
{
    // Should become a Binding0. 
    static Filter filterSysVars = new Filter() {
        public boolean accept(Object o)
        {
            Var v = (Var)o ; ;
            // Skip system variables.
            return Var.isSystemVar(v) ;
        }} ;
    
    public static Binding create() { return new BindingRoot() ; } 
    private BindingRoot() { super(null) ; }

    public boolean isRoot() { return true ; } 
    
    /** Iterate over all the names of variables.
     *  Mask the system variables and initial bindings.
     */

    public Iterator vars1()
    {
        return new FilterDropIterator(filterSysVars, super.vars1()) ; 
    }
    
    private boolean hasNonSysVar() { return vars1().hasNext() ; }

    protected void checkAdd1(Var var, Node node)
    { }
    
    public void format1(StringBuffer sBuff)
    {
        sBuff.append("[Root") ;
        
        if ( hasNonSysVar() )
        {
            sBuff.append(" ") ;
            super.format1(sBuff) ;    
        }
        
        sBuff.append("]") ;
    }
    
}

/*
 *  (c) Copyright 2004, 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
