/*
 * (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * [See end of file]
 */

package com.hp.hpl.jena.query.engine1.iterator;
import java.util.* ;

import org.apache.commons.logging.*;

import com.hp.hpl.jena.graph.*;
import com.hp.hpl.jena.graph.query.*;
import com.hp.hpl.jena.mem.GraphMemBaseQueryHandler;

import com.hp.hpl.jena.query.core.Var;
import com.hp.hpl.jena.query.engine.Binding;
import com.hp.hpl.jena.query.engine.BindingMap;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.QueryEngineUtils;
import com.hp.hpl.jena.query.engine1.plan.PlanBlockTriples;

import com.hp.hpl.jena.util.iterator.*;


/** An Iterator that takes a binding and executes a pattern.
 * 
 * @author     Andy Seaborne
 * @version    $Id: QueryIterBlockTriples.java,v 1.12 2007/01/02 11:19:31 andy_seaborne Exp $
 */
 
public class QueryIterBlockTriples extends QueryIterRepeatApply
{
    /** In raw mode, blank node variables are left in the binding */
    static public boolean rawMode = false ;
    // and they are in the alt matcher currently They get hidden by projection anyway.
    
    List pattern ;

    public static QueryIterator create( QueryIterator input,
                           List pattern , 
                           ExecutionContext cxt)
    {
        QueryHandler qh = cxt.getActiveGraph().queryHandler() ;
        if ( qh instanceof GraphMemBaseQueryHandler ||
             cxt.getContext().isTrueOrUndef(PlanBlockTriples.altMatcher) )
        {
            //LogFactory.getLog(QueryIterBlockTriples.class).warn("Altmatcher") ;
            // The alt matcher avoids thread creation - makes a difference when called very heavily.
            return new QueryIterBlockTripleAlt(input, pattern, cxt) ;
        }
            
        // The general matcher.
        return new QueryIterBlockTriples(input, pattern, cxt) ;
    }
    
    // TODO Make this private, have a static that chooses strategy
    
    private QueryIterBlockTriples( QueryIterator input,
                                  List pattern , 
                                  ExecutionContext cxt)
    {
        super(input, cxt) ;
        this.pattern = pattern ;
    }

    // **** No constraints done here currently
//    private QueryIterBasicPattern( QueryIterator input,
//                                  List pattern , List constraints, 
//                                  ExecutionContext cxt)
//    {
//        super(input, cxt) ;
//        this.pattern = pattern ;
//        this.constraints = constraints ;
//    }
     
    
    public QueryIterator nextStage(Binding binding)
    {
        return new StageBasePattern(binding, pattern, getExecContext()) ;
    }

    static class StageBasePattern extends QueryIter
    {
        ClosableIterator graphIter ;
        Binding binding ;
        //DatasetGraph data ;
        Node[] projectionVars ;

        // Could get pattern, constraints and data from parent if this were not static.
        // But non-static inner class that inherit from an external class can
        // be confusing.  Unnecessary complication.
        
        public StageBasePattern(Binding binding,
                                List pattern, 
                                ExecutionContext qCxt)
        {
            super(qCxt) ;
            this.binding = binding ;
            
            QueryHandler qh = qCxt.getActiveGraph().queryHandler() ;
            // In-memory one is com.hp.hpl.jena.mem.faster.GraphMemFasterQueryHandler
            // subclass of GraphMemBaseQueryHandler
            com.hp.hpl.jena.graph.query.Query graphQuery = new com.hp.hpl.jena.graph.query.Query() ;
            
            Set vars = new HashSet() ;
            
            QueryEngineUtils.compilePattern(graphQuery, pattern, binding, vars) ;
            projectionVars = QueryEngineUtils.projectionVars(vars) ; 
            // **** No constraints done here currently
            //QueryEngineUtils.compileConstraints(graphQuery, constraints) ;
            
            // Start our next iterator.
            BindingQueryPlan plan = qh.prepareBindings(graphQuery, projectionVars);
            graphIter = plan.executeBindings() ;
            if ( graphIter == null )
                LogFactory.getLog(StageBasePattern.class).warn("Graph Iterator is null") ;
        }

        //@Override
        protected boolean hasNextBinding()
        {
            boolean isMore = graphIter.hasNext() ;
            return isMore ;
        }

        //@Override
        protected Binding moveToNextBinding()
        {
          Domain d = (Domain)graphIter.next() ;
          Binding b = graphResultsToBinding(binding, d, projectionVars) ;
          return b ;
        }

        //@Override
        protected void closeIterator()
        {
            if ( ! isFinished() )
            {
                if ( graphIter != null )
                    graphIter.close() ;
                graphIter = null ;
            }
        }
        
    }
    
    private static Binding graphResultsToBinding(Binding parent, Domain d, Node[] projectionVars)
    {
        // Copy out (into a smaller structure).
        // Mask bnode variables.
        Binding binding = new BindingMap(parent) ;
        
        for ( int i = 0 ; i < projectionVars.length ; i++ )
        {
            String name = projectionVars[i].getName() ;
            if ( ! rawMode && Var.isBlankNodeVarName(name) )
                // Blank node variables don't get out.
                continue ;
            
            Node n = (Node)d.get(i) ;
            if ( n == null )
                // There was no variable of this name.
                continue ;
            binding.add(Var.alloc(name), n) ;
        }
        return binding ;
    }
}

/*
 *  (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
