/*
 * (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.query.function;

import java.util.List ;

import com.hp.hpl.jena.query.engine.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.expr.NodeValue ;

/** Interface to value-testing extensions to the expression evaluator.
 * 
 * @author Andy Seaborne
 * @version $Id: Function.java,v 1.16 2007/01/02 11:20:49 andy_seaborne Exp $
 */

public interface Function
{
    /* Richard:
    The Function interface should be implementable without knowing much about the rest of ARQ. 
    The javadocs are too concise. I think they should mention:
        - that build() can be useful for checking the number of arguments (anything else?)
        - what exception should be thrown by build() if bad arguments
        - that the URI parameter of build() is just the name of the function/ extension
        - the class of the items in the args List
        - how exec() should deal with bad input data (return null? throw some
        exception?)
   */
    
    
    /** Called during query plan construction immediately after the
     * construction of the extension instance.
     * Can throw ExprBuildException if something is wrong (like wrong number of arguments). 
     * @param args The parsed arguements
     */ 
    public void build(String uri, List args) ;

    /** Test a list of values - argument will not be null but
     *  may have the wrong number of arguments.
     *  FunctionBase provides a more convenient way to implement a function. 
     *  Can throw ExprEvalsException if something goes wrong.
     *   
     * @param binding   The current solution
     * @param args      A list of unevaluated expressions
     * @param uri       The name of this   
     * @param execCxt   The execution context
     * @return NodeValue - a value
     */ 
    
    public NodeValue exec(Binding binding, List args, String uri, ExecutionContext execCxt) ;
}

/*
 * (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */