/*
 * (c) Copyright 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.query.pfunction;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.expr.Expr;
import com.hp.hpl.jena.query.extension.Extension;
import com.hp.hpl.jena.query.extension.ExtensionRegistry;
import com.hp.hpl.jena.query.util.ExprUtils;
import com.hp.hpl.jena.vocabulary.RDFS;

/** Map from the new property function implementation to old-style
 * extensions of 2 arguments.  Temporary adpater.
 * 
 * @author Andy Seaborne
 * @version $Id: PropFuncAdapter.java,v 1.2 2007/01/02 11:20:27 andy_seaborne Exp $
 */

// Called from PlanBasicgraphPattern (isMagicProperty) and PlanPropertyFunction (constructor) 

public class PropFuncAdapter implements PropertyFunction
{
    private Log log = LogFactory.getLog(PropFuncAdapter.class) ;
    Extension ext = null ;
    List args ;
    
    // Force registration
    static {
//        ExtensionRegistry.get().put(ListMember.member.getURI(),
//                                    com.hp.hpl.jena.query.extension.library.list.class) ;
        ExtensionRegistry.get().put(RDFS.member.getURI(),
                                    com.hp.hpl.jena.query.extension.library.container.class) ;
    }

    public static boolean isMagicProperty(Triple t)
    {
        return ExtensionRegistry.get().isRegistered(t.getPredicate().getURI()) ;
    }
    
    public PropFuncAdapter(Extension ext)
    { 
        this.ext = ext ;
        log.info("Property function adapter for "+ext.getClass().getName()) ;
    }
    
    public void build(PropFuncArg argSubject, Node predicate, PropFuncArg argObject, ExecutionContext execCxt)
    {
        args = pfArgs(argSubject, argObject) ;
        ext.build(predicate.getURI(), args) ;
    }

    public QueryIterator exec(QueryIterator input,
                              PropFuncArg argSubject, Node predicate, PropFuncArg argObject,
                              ExecutionContext execCxt)
    {
        return ext.exec(input, args, predicate.getURI(), execCxt) ;
    }

    private List pfArgs(PropFuncArg argSubject, PropFuncArg argObject)
    {
        List args = new ArrayList() ; // expressions
        
        Expr sExpr = ExprUtils.nodeToExpr(argSubject.getArg()) ;
        Expr oExpr = ExprUtils.nodeToExpr(argObject.getArg()) ;
        
        args.add(sExpr) ;
        args.add(oExpr) ;
        
        return args ;
    }
}

/*
 * (c) Copyright 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */