/*
 * (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.query.util;

import java.util.HashMap;
import java.util.Map;
import org.apache.commons.logging.LogFactory;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.ARQInternalErrorException;

/** Map from bNode to _:* form. This is not a global mapping - it's "per map".
 * 
 * @author Andy Seaborne
 * @version $Id: NodeToLabelMap.java,v 1.10 2007/01/02 11:20:04 andy_seaborne Exp $
 */

public class NodeToLabelMap
{
    int bNodeCounter = 0 ;
    Map bNodeStrings = new HashMap() ;
    boolean bNodesAsFakeURIs = false ;
    String prefixString = "b" ;
    
    public NodeToLabelMap() { this("b", false) ; }
    
    public NodeToLabelMap(String prefix, boolean bNodesAsFakeURIs)
    {
        this.bNodesAsFakeURIs = bNodesAsFakeURIs ;
        this.prefixString = "_:"+prefix ;
    }
    
    public String asString(Node n)
    {
        if ( ! n.isBlank() && ! n.isVariable() )
        {
            LogFactory.getLog(NodeToLabelMap.class).warn("Not a variable or bNode: "+n) ;
            return "" ;
        }

        
        String s = (String)bNodeStrings.get(n) ;
        if ( s != null )
            return s ;
        
        if ( bNodesAsFakeURIs && n.isBlank() )
            s = "<_:"+n.getBlankNodeId().getLabelString()+">" ;
        else
        {
            s = prefixString+(bNodeCounter++) ;
            // Could put the fake URIs in - but we don't
            bNodeStrings.put(n, s) ;
        }
        return s ;
    }

    /**
     * @return Returns the bNodesAsFakeURIs.
     */
    public boolean isBNodesAsFakeURIs()
    {
        return bNodesAsFakeURIs ;
    }

    /**
     * @param nodesAsFakeURIs The bNodesAsFakeURIs to set.
     */
    public void setBNodesAsFakeURIs(boolean nodesAsFakeURIs)
    {
        bNodesAsFakeURIs = nodesAsFakeURIs ;
    }

    /**
     * @return Returns the prefix.
     */
    public String getPrefixString()
    {
        return prefixString ;
    }

    /**
     * @param prefix The prefix to set.
     */
    public void setPrefixString(String prefix)
    {
        if ( prefix == null )
        {
            LogFactory.getLog(NodeToLabelMap.class).fatal("Prefix string is null") ;
            throw new ARQInternalErrorException("Prefix string is null") ;
        }
        if ( prefix.equals("") )
        {
            LogFactory.getLog(NodeToLabelMap.class).fatal("Prefix string is the empty string") ;
            throw new ARQInternalErrorException("Prefix string is the empty string") ;
        }
            
        this.prefixString = prefix ;
    }
}

/*
 * (c) Copyright 2005, 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */