/*
 * (c) Copyright 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.query.util;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.engine.Plan;
import com.hp.hpl.jena.query.engine.QueryEngineBase;
import com.hp.hpl.jena.query.engine2.QueryEngineQuad;
import com.hp.hpl.jena.query.engine2.QueryEngineRef;
import com.hp.hpl.jena.query.serializer.SerializationContext;

/** Utilities for queries and plans
 * 
 * @author Andy Seaborne
 * @version $Id: QueryUtils.java,v 1.16 2007/01/06 13:56:50 andy_seaborne Exp $
 */

public class QueryUtils
{
    public static void printPlan(Query query)
    {
        QueryExecution qe = QueryExecutionFactory.create(query) ;
        _print(query, qe) ;
    }
    
    public static void printOp(Query query)
    {
        QueryExecution qe = new QueryEngineRef(query) ;
        _print(query, qe) ;
    }    

    public static void printQuad(Query query)
    {
        QueryExecution qe = new QueryEngineQuad(query) ;
        _print(query, qe) ;
    }    
    
    private static void _print(Query query, QueryExecution qe)
    {
        if ( qe instanceof QueryEngineBase )
        {
            Plan plan = ((QueryEngineBase)qe).getPlan() ;
            SerializationContext sCxt = new SerializationContext(query) ;
            IndentedWriter out = new IndentedWriter(System.out) ;
            plan.output(out, sCxt) ;
            out.flush();
            return ;
        }
        System.err.println("printPlan: Unknown engine type: "+Utils.className(qe)) ;
    }
}

/*
 * (c) Copyright 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */